/*
 *  Types/Note.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using System.Reflection;
using Incollector.GUI.Widgets;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class SourceCode : DataType {
		
		protected static Gtk.Entry    TitleWidget = new Gtk.Entry ();
		protected static Gtk.Entry    LanguageWidget = new Gtk.Entry ();
		protected static Gtk.TextView CodeWidget = new Gtk.TextView ();
		
		protected string _DataTitle = "";
		protected string _DataLanguage = "";
		protected string _DataCode = "";
		
		
		protected string DataTitle {
			get { return this._DataTitle;  }
			set { this._DataTitle = value; }
		}
		
		protected string DataCode {
			get { return this._DataCode;  }
			set { this._DataCode = value; }
		}
		
		protected string DataLanguage {
			get { return this._DataLanguage;  }
			set { this._DataLanguage = value; }
		}
		
		
		
		public override string Name {
			get { return "sourcecode"; }
		}
		
		public override string Title {
			get { return Catalog.GetString ("Source code"); }
		}
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a source code"); }
		}
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return TitleWidget; }
		}
		
		public override Gtk.Widget FocusWidget {
			get { return TitleWidget; }
		}
		
		public override Gtk.Widget EditWidget {
			get {
				Gtk.Table tbl = new Gtk.Table (2, 2, false);
				
				Gtk.Label lbl0 = new Gtk.Label (Catalog.GetString ("Title"));
				lbl0.Xalign = 1;
				lbl0.Yalign = (float) 0.50;
				tbl.Attach (lbl0, 0, 1, 0, 1, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				TitleWidget = new Gtk.Entry ();
				TitleWidget.Text = this._DataTitle;
				
				tbl.Attach (TitleWidget, 1, 2, 0, 1,
							AttachOptions.Shrink | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Language"));
				lbl1.Xalign = 1;
				lbl1.Yalign = (float) 0.50;
				tbl.Attach (lbl1, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				LanguageWidget = new Gtk.Entry ();
				LanguageWidget.Text = this._DataLanguage;
				Incollector.Engine.Autocompletion.Assign (LanguageWidget, "data:" +
															Name + ":language");
				
				tbl.Attach (LanguageWidget, 1, 2, 1, 2,
							AttachOptions.Shrink | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Code"));
				lbl2.Xalign = 1;
				lbl2.Yalign = 0;
				tbl.Attach (lbl2, 0, 1, 2, 3, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				CodeWidget = new Gtk.TextView ();
				CodeWidget.Buffer.Text = this._DataCode;
				CodeWidget.HeightRequest = 150;
				CodeWidget.AcceptsTab = false;
				CodeWidget.WrapMode = Gtk.WrapMode.Word;
				CodeWidget.PixelsAboveLines = 1;
				CodeWidget.LeftMargin = 3;
				CodeWidget.RightMargin = 3;
				
				Pango.FontDescription fnt = new Pango.FontDescription ();
				if (Engine.FilesPaths.IsWinSystem () == true)
					fnt.Family = "Courier New";
				else
					fnt.Family = "Monospace";
				
				CodeWidget.ModifyFont (fnt);
				
				Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
				scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
				scroll.ShadowType = Gtk.ShadowType.In; 
				scroll.Add (CodeWidget);
				tbl.Attach (scroll, 1, 2, 2, 3,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Expand | AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();
				return tbl;
			}
		}
		
		public override string EditWidgetContent {
			set { this.DataCode = value; }
		}
		
		public override Gtk.Widget PreviewWidget {
			get {
				Pango.FontDescription fnt = new Pango.FontDescription ();
				if (Engine.FilesPaths.IsWinSystem () == true)
					fnt.Family = "Courier New";
				else
					fnt.Family = "Monospace";
				
				
				Gtk.Table tbl = new Gtk.Table (2, 2, false);
				
				Gtk.Label lbl0 = new Gtk.Label ();
				lbl0.UseMarkup = true;
				lbl0.Markup    = "<b>" + Catalog.GetString ("Title") + "</b>";
				lbl0.Xalign    = 1;
				tbl.Attach (lbl0, 0, 1, 0, 1, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel Title  = new PreviewLabel (this.DataTitle);
				Title.Xalign     = 0;
				Title.Yalign     = 0;
				tbl.Attach (Title, 1, 2, 0, 1,AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl1 = new Gtk.Label ();
				lbl1.UseMarkup = true;
				lbl1.Markup    = "<b>" + Catalog.GetString ("Language") + "</b>";
				lbl1.Xalign    = 1;
				tbl.Attach (lbl1, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel Language  = new PreviewLabel (this.DataLanguage);
				Language.Xalign     = 0;
				Language.Yalign     = 0;
				tbl.Attach (Language, 1, 2, 1, 2,AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl2 = new Gtk.Label ();
				lbl2.UseMarkup = true;
				lbl2.Markup    = "<b>" + Catalog.GetString ("Source code") + "</b>";
				lbl2.Xalign    = 1;
				lbl2.Yalign    = 0;
				tbl.Attach (lbl2, 0, 1, 2, 3, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				PreviewLabel SourceCode = new PreviewLabel ("\n" + this.DataCode);
				SourceCode.ModifyFont (fnt);
				SourceCode.Wrap = true;
				SourceCode.Xalign = 0;
				SourceCode.Yalign = 0;
				tbl.Attach (SourceCode, 1, 2, 2, 3, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();

					
				return tbl;
			}
		}
		
		public override string EntryTitle {
			get {
				// TODO: Remove it before 1.0 release
				if (this.DataTitle != "")
					return this.DataTitle;
				else
					return this.DataCode;
			}
		}
		
		public override Gdk.Pixbuf Icon16 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "sourcecode-16.png"); }
		}
		
		public override Gdk.Pixbuf Icon32 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "sourcecode-32.png"); }
		}
		
		public override Gdk.Pixbuf Icon48 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "sourcecode-48.png"); }
		}
		
		
		
		public SourceCode ()
		{
		}
		
		
		public override DataType New ()
		{
			return new SourceCode ();
		}
		
		
		public override DataType Clone ()
		{
			SourceCode src = new SourceCode ();
			
			src._DataCode = this._DataCode;
			src._DataLanguage = this._DataLanguage;
			src._DataTitle = this._DataTitle;
			
			return src;
		}


		public override void ReadXml (XmlTextReader reader)
		{
			this.DataTitle    = reader.GetAttribute ("title");
			this.DataLanguage = reader.GetAttribute ("language");
			this.DataCode     = reader.ReadString ();
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", this.DataLanguage);
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("title", this.DataTitle);
				writer.WriteAttributeString ("language", this.DataLanguage);
				writer.WriteString (this.DataCode);
			writer.WriteEndElement ();
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			LanguageWidget.Changed += hdlr;
			TitleWidget.Changed += hdlr;
			CodeWidget.Buffer.Changed += hdlr;
		}
		
		
		public override bool Search (string query)
		{
			return Engine.Text.Search (query, this.DataTitle) ||
							Engine.Text.Search (query, this.DataLanguage) ||
							Engine.Text.Search (query, this.DataCode);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (TitleWidget.Text != "") && (LanguageWidget.Text != "") &&
						(CodeWidget.Buffer.Text != "");
		}
		
		
		public override void SynchronizeData ()
		{
			this._DataCode = CodeWidget.Buffer.Text;
			this._DataLanguage = LanguageWidget.Text;
			this._DataTitle = TitleWidget.Text;
		}
	}
}
